<?php /**
 * @file
 * Contains \Drupal\ark\Controller\DefaultController.
 */

namespace Drupal\ark\Controller;

use Drupal\Core\Controller\ControllerBase;

/**
 * Default controller for the ark module.
 */
class DefaultController extends ControllerBase {

  public function ark_info_page() {
    return [
      '#markup' => t('<p>This site, %site_name, implements Archival Resource Key URLs designed to support long-term access to information. It is a Name Mapping Authority Hostport for the Naming Assigning Authority Number %naan.</p><p>Read more about <a href="@ark_url">ARK</a>.</p><p>If you reached this page instead of an expected resource, be sure you have "ark:/", with the colon, in the URL.', [
        '%site_name' => variable_get('site_name', 'Drupal'),
        '%naan' => variable_get('ark_naan', ''),
        '@ark_url' => 'https://wiki.ucop.edu/display/Curation/ARK',
        '@arguments_link' => url('examples/page_example/arguments/23/56', [
          'absolute' => TRUE
          ]),
      ])
      ];
  }

  public function ark_redirect($naan, $name) {
    if (!ark_naan_format_valid($naan)) {
      drupal_add_http_header('Status', '404 Not Found');
      $this->messenger()->addError(t('The Name Assigning Authority Number (NAAN) must be a number'));
      return '';
    }
    if ($naan != variable_get('ark_naan', 'pending')) {
      drupal_add_http_header('Status', '404 Not Found');
      $this->messenger()->addError(t('This site only services requests for Archival Resource Keys under the Name Assigning Authority Number <strong>@naan</strong>.', [
        '@naan' => variable_get('ark_naan', 'pending')
        ]));
      return '';
    }
    $ark_name = ark_name_strip_invalid_characters($name);
    $entity = ark_resource_load($ark_name);
    if (!$entity) {
      drupal_add_http_header('Status', '404 Not Found');
      drupal_set_title('No resource found');
      $this->messenger()->addError(t('No resource found for Archival Resource Key name %ark_name under the Name Assigning Authority Number <strong>@naan</strong>.', [
        '%ark_name' => $ark_name,
        '@naan' => variable_get('ark_naan', 'pending'),
      ]));
      return '';
    }
    $uri = entity_uri($entity->entity_type, $entity);
    drupal_goto(url($uri['path'], $uri['options']));

  }

  public function ark_malformed($naan) {
    if (!ark_naan_format_valid($naan)) {
      $this->messenger()->addError(t('The Name Assigning Authority Number (NAAN) must be a number'));
    }
    drupal_add_http_header('Status', '404 Not Found');
    $this->messenger()->addError(t('The ARK request is malformed.  It must have a name component.'));
    return '';
  }

}
