<?php

namespace Drupal\ark\Form;

use Drupal\ark\ArkEnabledContentTypeInformation;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ArkEnabledContentTypeEditForm
 *
 * The content citation mapping edit form.
 *
 * @package Drupal\ark\Form
 */
class ArkEnabledContentTypeEditForm extends EntityForm {

  /**
   * The citation map information service.
   *
   * @var \Drupal\ark\ArkEnabledContentTypeInformation
   */
  protected $arkContentTypeInfo;

  /**
   * Constructs a ArkEnabledContentTypeEditForm object.
   *
   * @param \Drupal\ark\ArkEnabledContentTypeInformation $ark_content_type_info
   *   The content citation mapping information service.
   */
  public function __construct(ArkEnabledContentTypeInformation $ark_content_type_info) {
    $this->arkContentTypeInfo = $ark_content_type_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ark.content_type_info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    /** @var \Drupal\ark\Entity\ArkEnabledContentType $entity */
    $entity = $this->entity;

    $form['#title'] = $this->t('Configure %label', ['%label' => $entity->label()]);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#disabled' => !$entity->isNew(),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\ark\Entity\ArkEnabledContentType::load',
      ],
      '#disabled' => !$entity->isNew(),
    ];

    // $trying_in_cli = \Drupal::service('ark.content_type_info')->getContentFieldOptions('node', 'pece_essay', 'datetime');
    $when_options = $this->arkContentTypeInfo->getContentFieldOptions($entity->getTargetEntityTypeId(), $entity->getTargetBundle(), 'datetime');
    $when_default = $entity->getWhenField();
    $form['when_field'] = [
      '#title' => $this->t('When field'),
      '#type' => 'select',
      // @TODO make the fallback default values not hard-coded.
      '#default_value' => $when_default ?? 'field_created_date',
      '#options' => $when_options,
    ];

    // @TODO maybe take multiple types of fields here and allow string fields also.
    $who_options = $this->arkContentTypeInfo->getContentFieldOptions($entity->getTargetEntityTypeId(), $entity->getTargetBundle(), 'entity_reference');
    $who_default = $entity->getWhoFields();
    $form['who_fields'] = [
      '#title' => $this->t('Who fields'),
      '#type' => 'select',
      '#multiple' => TRUE,
      '#default_value' => $who_default ?? ['field_people_with_edit_access'],
      '#options' => $who_options,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\ark\Entity\ArkEnabledContentType $entity */
    return parent::save($form, $form_state);
    $entity = $this->entity;

    $status = $entity->save();

    switch ($status) {
      case SAVED_NEW:
        // This has always happened already on the previous step, but tradition.
        $this->messenger()->addMessage($this->t('Created the @label.', [
          '@label' => $entity->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the @label.', [
          '@label' => $entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($entity->toUrl('collection'));

    return $status;
  }

  /**
   * Provides the edit title.
   *
   * Note that this only affects the breadcrumb. The main title is set in the
   * form itself.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return string|null
   *   The title for the entity edit page, if an entity was found.
   */
  public function getTitle(RouteMatchInterface $route_match) {
    $entity = $route_match->getParameter('ark_content_type');
    return $this->t('%label', ['%label' => $entity->label()]);
  }

}
