<?php

/**
 * @file
 * Contains \Drupal\ark\Form\ArkRemoteUpdateSettingsForm.
 */

namespace Drupal\ark\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class ArkRemoteUpdateSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ark_remote_update_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ark.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ark.settings'];
  }

  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['ezid_remote_status_explanation'] = [
      '#type' => 'fieldset',
      '#title' => t('EZID Remote Status field'),
      '#weight' => 0,
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#description' => '<div>
        <h2>' . t('Identifier status') . '</h2>
        <p>' . t('Each identifier in EZID has a status.  The status is recorded as the value of the "_status" reserved metadata element (see <a class="reference internal" href="https://ezid.cdlib.org/doc/apidoc.html#internal-metadata">Internal metadata</a> on EZid site) and may be one of:') . '</p>
        <dl>
          <dt><strong>' . t('public') . '</strong></dt>
          <dd>' . t('The default value.') . '</dd>
          <dt><strong>' . t('reserved') . '</strong></dt>
          <dd>' . t('The identifier is known only to EZID.  This status may be used to reserve an identifier name within EZID without advertising the identifier\'s existence to resolvers and other external services. A reserved identifier may be deleted.') . '</dd>
          <dt><strong>' . t('unavailable') . '</strong></dt>
          <dd>' . t('The identifier is public, but the object referenced by the identifier is not available.  A reason for the object\'s unavailability may optionally follow the status separated by a pipe character ("|", U+007C), e.g., "unavailable | withdrawn by author".') . t('The identifier redirects to an EZID-provided "tombstone" page (an HTML page that displays the identifier\'s citation metadata and the reason for the object\'s unavailability) regardless of its target URL.') . '</dd>
        </dl>
        <p>' . t('An identifier\'s status may be changed by setting a new value for the aforementioned "_status" metadata element.  EZID permits only certain status transitions:') . '</p>
        <ul>
          <li>' . t('A status of "reserved" may be specified only at identifier creation time.') . '</li>
          <li>' . t('A reserved identifier may be made public.  At this time the identifier will be registered with resolvers and other external services.') . '</li>
          <li>' . t('A public identifier may be marked as unavailable.  At this time the identifier will be removed from resolvers and external services to the extent possible.') . '</li>
          <li>' . t('An unavailable identifier may be returned to public status.  At this time the identifier will be re-registered with resolvers and other external services.') . '</li>
        </ul>
      </div>',
    ];

    $form['ezid_remote_status_explanation']['ezid_remote_status'] = [
      '#type' => 'textfield',
      '#title' => t('The default value for status field when creating a new remote entry.'),
      '#description' => '<strong>' . t('PLEASE DO NOT CHANGE THIS VALUE IF YOU DON\'T KNOW WHAT IT MEANS!!!') . '</strong>',
      '#default_value' => variable_get('ezid_remote_status', ARK_DEFAULT_REMOTE_STATUS_ENTRY_VALUE),
      '#size' => 30,
      '#required' => TRUE,
    ];

    $form['ezid_remote_status_explanation']['ark_ezid_remote_export'] = [
      '#type' => 'checkboxes',
      '#title' => t('Determines if the identifier is publicized by exporting it to external indexing and harvesting services. Default is checked.'),
      '#description' => '<strong>' . t('PLEASE DO NOT CHANGE THIS VALUE IF YOU DON\'T KNOW WHAT IT MEANS!!!') . '</strong>',
      '#options' => [
        'yes' => t('Allow harvesting/indexing.')
        ],
      '#default_value' => variable_get('ark_ezid_remote_export', ARK_DEFAULT_REMOTE_EXPORT_ENTRY_VALUE),
    ];

    $form['ezid_remote_status_explanation']['ark_ezid_update_if_exists'] = [
      '#type' => 'checkboxes',
      '#title' => t('Update remote ARK entry if already exist. If not, create a new one.'),
      '#options' => [
        '1' => t('Update remote ARK entry if already exist. If not, create a new one.')
        ],
      '#default_value' => variable_get('ark_ezid_update_if_exists', ARK_DEFAULT_REMOTE_UPDATE_IF_EXISTS),
    ];

    return parent::buildForm($form, $form_state);
  }

}
?>
