<?php

/**
 * @file
 * Contains \Drupal\ark\Form\PeceArkBatchesForm.
 */

namespace Drupal\ark\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class PeceArkBatchesForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pece_ark_batches_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['description'] = [
      '#type' => 'markup',
      '#markup' => t('Select the batch operation'),
    ];
    $form['batch'] = [
      '#type' => 'select',
      '#title' => 'Choose batch',
      '#options' => [
        'pece_ark_batch_for_local_ark_data_creation' => t('Generate Ark data for all ARK content types selected'),
        'pece_ark_batch_for_remote_data_update' => t('Update Ark remote data for all ARK content types selected'),
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'Go',
    ];

    // If no nodes, prevent submission.
    // Find out if we have a node to work with. Otherwise it won't work.
    $nids = _get_all_node_ids_for_selected_ark_node_types();
    $remote_status = variable_get('ark_ezid_remote_status', ARK_DEFAULT_REMOTE_STATUS_ENTRY_VALUE);
    $username = variable_get('ark_ezid_user');
    $password = variable_get('ark_ezid_password');
    if (empty($nids)) {
      $this->messenger()->addWarning(t("You currently don't have any nodes for the selected ARK content types, and this batch requires at least one node to be processed. As a result, this form is disabled."));
      $form['submit']['#disabled'] = TRUE;
    }
    if (empty($username)) {
      $this->messenger()->addWarning(t("You have not set a username for connecting to the remote sever under the ") . l(t('Ark Module Settings page'), '/admin/config/services/ark/ark') . ". As a result, this form is disabled.");
      $form['submit']['#disabled'] = TRUE;
    }
    if (empty($password)) {
      $this->messenger()->addWarning(t("You have not set a password for connecting to the remote sever under the ") . l(t('Ark Module Settings page'), '/admin/config/services/ark/ark') . ". As a result, this form is disabled.");
      $form['submit']['#disabled'] = TRUE;
    }
    if (empty($remote_status)) {
      $this->messenger()->addWarning(t("You have not set a default status for new entries to the remote sever under the ") . l(t('Ark Module Settings page'), 'admin/config/services/ark/remote_update_settings') . ". As a result, this form is disabled.");
      $form['submit']['#disabled'] = TRUE;
    }
    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $function = $form_state->getValue(['batch']);

    if ($form_state->getValue(['batch']) == 'pece_ark_batch_for_remote_data_update') {
      if (empty(variable_get('ark_ezid_user')) || empty(variable_get('ark_ezid_password'))) {
        $form_state->setErrorByName('batch', t('Values for ark_ezid_user or ark_ezid_password may be empty. Please set it in the ') . l(t('Ark Module Settings page'), '/admin/config/services/ark/ark')) ;
      }
    }

    // Reset counter for debug information.
    $_SESSION['http_request_count'] = 0;
    $batch = $function();

    batch_set($batch);
  }

}
?>
