<?php

/**
 * @file
 * Contains \Drupal\ark\Form\PeceArkSettingsForm.
 */

namespace Drupal\ark\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class PeceArkSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pece_ark_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ark.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ark.settings'];
  }

  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = [];

    $form['ark-form-settings'] = [
      '#type' => 'fieldset',
      '#title' => t('Setup Ark Module settings'),
      '#weight' => 0,
      '#description' => t('Set the parameters for the Ark Module settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];

    $form['ark-form-settings']['ark_composite_who_field'] = [
      '#type' => 'textfield',
      '#size' => 65,
      '#title' => t('Set the machine name of the fields that will compose the "who" field. Pelase add the fields separated by comma.'),
      '#default_value' => variable_get('ark_composite_who_field', implode(',', ARK_COMPOSITE_WHO_FIELD)),
      '#maxlength' => 1024,
      '#required' => TRUE,
      '#description' => implode(',', ARK_COMPOSITE_WHO_FIELD),
    ];

    $form['ark-form-settings']['ark_indexing_limit'] = [
      '#type' => 'textfield',
      '#size' => 65,
      '#title' => t('Set the limit of nodes to be processed at the same time during the bach process for crating local and remote ARK ID\'s.'),
      '#default_value' => variable_get('ark_indexing_limit', ARK_INDEXING_LIMIT),
      '#maxlength' => 1024,
      '#required' => TRUE,
      '#description' => 'Ex.: ' . ARK_INDEXING_LIMIT,
    ];

    $form['ark-form-settings']['ark_authors_name_separator'] = [
      '#type' => 'textfield',
      '#size' => 65,
      '#title' => t('Set the separator simbol to delimit the authors names.'),
      '#default_value' => variable_get('ark_authors_name_separator', ARK_DEFAULT_AUTHORS_NAME_SEPARATOR),
      '#maxlength' => 1024,
      '#required' => TRUE,
      '#description' => 'Ex.: ' . ARK_DEFAULT_AUTHORS_NAME_SEPARATOR,
    ];

    return parent::buildForm($form, $form_state);
  }

}
?>
